unit chatserv;

interface

uses
  Classes, blcksock, synsock, Sysutils, SyncObjs, synautil;

type

  { TTCPChatDaemon }

  TTCPChatDaemon = class(TThread)
  private
    fPort: string;
    Sock: TTCPBlockSocket;
  public
    constructor Create;
    destructor Destroy; override;
    procedure Execute; override;
    property Port: string read fPort write fPort;
  end;

  TTCPChatThrd = class(TThread)
  private
    Sock: TTCPBlockSocket;
    CSock: TSocket;
  public
    constructor Create(hsock: tSocket);
    procedure Execute; override;
  end;

implementation

var Chat: TStringList;
var CS: TCriticalSection;

{ TChatDaemon }

constructor TTCPChatDaemon.Create;
begin
  Chat := TStringList.Create;
  sock := TTCPBlockSocket.create;
  FreeOnTerminate := true;
  inherited create(true);
end;

destructor TTCPChatDaemon.Destroy;
begin
  Sock.free;
  Chat.Free;
end;

procedure TTCPChatDaemon.Execute;
var
  ClientSock: TSocket;
begin
  with sock do
  begin
    CreateSocket;
    setLinger(true, 10);
    EnableReuse(true);
    bind('0.0.0.0', Port);
    listen;
    repeat
      if terminated then break;
      if canread(1000) then
      begin
        ClientSock := accept;
        if lastError = 0 then TTCPChatThrd.create(ClientSock);
      end;
    until false;
  end;
end;

{ TEchoThrd }

constructor TTCPChatThrd.Create(Hsock: TSocket);
begin
  Csock := Hsock;
  FreeOnTerminate := true;
  inherited create(false);
end;

procedure TTCPChatThrd.Execute;
var
  s: string;
  Count: integer;
begin
  sock := TTCPBlockSocket.create;
  try
    Sock.socket := CSock;
    sock.GetSins;
    with sock do
    begin
      s := RecvPacket(60000);

      if lastError = 0 then
      begin
        if s = 'insert' then
        begin
          SendString(#13#10);
          s := RecvPacket(60000);
          CS.Enter;
          Chat.Insert(0, s);
          CS.Leave;
        end;

        if SeparateLeft(s, '_') = 'list' then
        begin
          Count := StrToInt(SeparateRight(s, '_'));
          SendString(#13#10);
          if count < Chat.count then
          begin
           CS.Enter;
           S:= Chat.Text + #13#10;
           CS.Leave;
           SendString(S);
          end else SendString(#13#10);
        end;

      end;
    end;
  finally
    Sock.Free;
  end;
end;

initialization

  CS := TCriticalSection.Create;

finalization

  CS.free;

end.
